@@ -9,6 +9,7 @@ class ProtoWrapper:
         self.result = pb2.Result()
         self.scan_meta = pb2.ScanMeta()
         self.bugs_list = []
+        self.assets_list = []
         self.result.analyzer = getattr(pb2.Result.Analyzer, analyzer.upper())
         self.report = None
 
@@ -20,6 +21,7 @@ class ProtoWrapper:
 
     def generate_report(self):
         self.result.bugs_list.extend(self.bugs_list)
+        self.result.assets_list.extend(self.assets_list)
         self.result.scan_meta.CopyFrom(self.scan_meta)
         self.report = MessageToDict(
             self.result, including_default_value_fields=True)
@@ -35,6 +37,8 @@ class ProtoWrapper:
             sast.start_line_no = kwargs.get("start_line_no")
         if "end_line_no" in kwargs:
             sast.end_line_no = kwargs.get("end_line_no")
+        if "commit" in kwargs:
+            sast.commit = kwargs.get("commit")
         return sast
 
     @staticmethod
@@ -46,14 +50,20 @@ class ProtoWrapper:
                 dast.uri.extend(uri)
         if "port" in kwargs:
             port = kwargs.get("port")
-            if isinstance(port, int):
-                dast.port = port
+            if isinstance(port, dict):
+                port = ProtoWrapper.create_port_object(**port)
+            if isinstance(port, pb2.Port):
+                dast.port.CopyFrom(port)
         if "method" in kwargs:
             dast.method = kwargs.get("method")
         if "param" in kwargs:
             dast.param = kwargs.get("param")
         if "evidence" in kwargs:
             dast.evidence = kwargs.get("evidence")
+        if "request_headers" in kwargs:
+            dast.request_headers = kwargs.get("request_headers")
+        if "response_headers" in kwargs:
+            dast.response_headers = kwargs.get("response_headers")
         return dast
 
     @staticmethod
@@ -62,7 +72,11 @@ class ProtoWrapper:
         if "ipaddress" in kwargs:
             network.ipaddress = kwargs.get("ipaddress")
         if "port" in kwargs:
-            network.port = kwargs.get("port")
+            port = kwargs.get("port")
+            if isinstance(port, dict):
+                port = ProtoWrapper.create_port_object(**port)
+            if isinstance(port, pb2.Port):
+                network.port.CopyFrom(port)
         if "hostname" in kwargs:
             network.hostname = kwargs.get("hostname")
         if "macaddress" in kwargs:
@@ -80,6 +94,8 @@ class ProtoWrapper:
             package.installed_version = kwargs.get("installed_version")
         if "affected_versions" in kwargs:
             package.affected_versions = kwargs.get("affected_versions")
+        if "commit" in kwargs:
+            package.commit = kwargs.get("commit")
         return package
 
     @staticmethod
@@ -92,7 +108,11 @@ class ProtoWrapper:
         if "installed_version" in kwargs:
             container.installed_version = kwargs.get("installed_version")
         if "cpe" in kwargs:
-            container.cpe = kwargs.get("cpe")
+            cpes = kwargs.get("cpe")
+            if isinstance(cpes, list):
+                container.cpe.extend(cpes)
+        if "digest" in kwargs:
+            container.digest = kwargs.get("digest")
         return container
 
     @staticmethod
@@ -121,6 +141,65 @@ class ProtoWrapper:
             aws.category = kwargs.get("category")
         return aws
 
+    @staticmethod
+    def create_port_object(**kwargs) -> pb2.Port:
+        port = pb2.Port()
+        if "protocol" in kwargs:
+            port.protocol = kwargs.get("protocol")
+        if "port_id" in kwargs:
+            port.port_id = kwargs.get("port_id")
+        if "service" in kwargs:
+            port.service = kwargs.get("service")
+        if "product" in kwargs:
+            port.product = kwargs.get("product")
+        if "state" in kwargs:
+            state = kwargs.get("state")
+            if isinstance(state, int):
+                port.state = state
+        return port
+
+    @staticmethod
+    def create_asset_object(**kwargs) -> pb2.Asset:
+        asset = pb2.Asset()
+        if "name" in kwargs:
+            asset.name = kwargs.get("name")
+        if "ipaddress" in kwargs:
+            asset.ipaddress = kwargs.get("ipaddress")
+        if "os" in kwargs:
+            os = kwargs.get("os")
+            if isinstance(os, list):
+                asset.os.extend(os)
+        if "mac_address" in kwargs:
+            asset.mac_address = kwargs.get("mac_address")
+        if "hostname" in kwargs:
+            asset.hostname = kwargs.get("hostname")
+        if "netbios" in kwargs:
+            asset.netbios = kwargs.get("netbios")
+        if "cpe" in kwargs:
+            cpe = kwargs.get("cpe")
+            if isinstance(cpe, list):
+                asset.cpe.extend(cpe)
+        if "tags" in kwargs:
+            tags = kwargs.get("tags")
+            if isinstance(tags, list):
+                asset.tags.extend(tags)
+        if "risk_score" in kwargs:
+            asset.risk_score = kwargs.get("risk_score")
+        if "ports" in kwargs:
+            ports = kwargs.get("ports")
+            if isinstance(ports, list):
+                refined_ports = []
+                for port in ports:
+                    if isinstance(port, dict):
+                        port = ProtoWrapper.create_port_object(**port)
+                    refined_ports.append(port)
+                asset.ports.extend(refined_ports)
+        if "state" in kwargs:
+            state = kwargs.get("state")
+            if isinstance(state, int):
+                asset.state = state
+        return asset
+
     @staticmethod
     def create_bug_object(**kwargs) -> pb2.Bug:
         bug = pb2.Bug()
@@ -170,7 +249,7 @@ class ProtoWrapper:
                 bug.cloud.CopyFrom(cloud)
         if "package" in kwargs:
             package = kwargs.get("package")
-            if isinstance(package, pb2.Packgae):
+            if isinstance(package, pb2.Package):
                 bug.package.CopyFrom(package)
         if "container" in kwargs:
             container = kwargs.get("container")
@@ -185,13 +264,18 @@ class ProtoWrapper:
             if isinstance(extra_info, dict):
                 struct = Struct()
                 struct.update(extra_info)
-                bug.extra_info.CopyFrom(struct)
+            elif isinstance(extra_info, Struct):
+                struct = extra_info
+            bug.extra_info.CopyFrom(struct)
         if "scan_details" in kwargs:
             scan_details = kwargs.get("scan_details")
             if isinstance(scan_details, dict):
-                bug.scan_details = scan_details
-        if "assets" in kwargs:
-            assets = kwargs.get("assets")
-            if isinstance(assets, dict):
-                bug.assets = assets
+                for key, value in scan_details.items():
+                    bug.scan_details[key] = value
+        if "asset" in kwargs:
+            asset = kwargs.get("asset")
+            if isinstance(asset, pb2.Asset):
+                bug.asset.CopyFrom(asset)
+        if "vulnerable_since" in kwargs:
+            bug.vulnerable_since = kwargs.get("vulnerable_since")
         return bug
